@@ -1,4 +1,4 @@
-/*      $Id: xfwm4_plugin.c 16068 2005-06-25 21:51:21Z olivier $
+/*      $Id: xfwm4_plugin.c 18849 2005-11-14 21:05:29Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -2260,7 +2260,7 @@ write_options (McsPlugin * mcs_plugin)
 
     path = g_build_filename ("xfce4", "mcs_settings", RCFILE1, NULL);
     rcfile = xfce_resource_save_location (XFCE_RESOURCE_CONFIG, path, TRUE);    
-    if (G_LIKELY (rcfile == NULL))
+    if (G_LIKELY (rcfile != NULL))
     {
         result = mcs_manager_save_channel_to_file (mcs_plugin->manager, CHANNEL1, rcfile);
         g_free (rcfile);
@@ -2269,7 +2269,7 @@ write_options (McsPlugin * mcs_plugin)
 
     path = g_build_filename ("xfce4", "mcs_settings", RCFILE2, NULL);
     rcfile = xfce_resource_save_location (XFCE_RESOURCE_CONFIG, path, TRUE);    
-    if (G_LIKELY (rcfile == NULL))
+    if (G_LIKELY (rcfile != NULL))
     {
         result = mcs_manager_save_channel_to_file (mcs_plugin->manager, CHANNEL2, rcfile);
         g_free (rcfile);
