@@ -1,4 +1,4 @@
-/*      $Id: xfwm4_shortcuteditor.c 12448 2005-03-29 12:31:39Z pollux $
+/*      $Id: xfwm4_shortcuteditor.c 20610 2006-03-27 19:39:21Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -380,7 +380,7 @@ loadtheme_in_treeview (ThemeInfo *ti, gpointer data)
 
         if (g_str_has_prefix (*shortcut, "shortcut_") && g_str_has_suffix (*shortcut, "_exec"))
         {
-            *shortcut++;
+            shortcut++;
             continue;
         }
 
@@ -470,7 +470,7 @@ loadtheme_in_treeview (ThemeInfo *ti, gpointer data)
 	    gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, *shortcut, COLUMN_SHORTCUT, entry_value, -1);
 	}
 
-        *shortcut++;
+        shortcut++;
     }
 
     g_strfreev (shortcuts_list);
@@ -762,7 +762,7 @@ cb_compose_shortcut (GtkWidget * widget, GdkEventKey * event, gpointer data)
             strcat (shortcut_string, *current_shortcut);
             strcat (shortcut_string, "+");
         }
-        *current_shortcut++;
+        current_shortcut++;
     }
      
     shortcut_string[strlen (shortcut_string) - 1] = '\0'; /* replace the trailing '+' */
