@@ -1,4 +1,4 @@
-/*      $Id: compositor.c 18139 2005-10-08 19:43:18Z olivier $
+/*      $Id: compositor.c 22177 2006-06-22 13:01:37Z olivier $
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -1030,14 +1030,11 @@ paint_all (ScreenInfo *screen_info, XserverRegion region)
         }
     }
 
+    TRACE ("Copying data back to screen");
     XFixesDestroyRegion (dpy, region);
-    if (screen_info->rootBuffer != screen_info->rootPicture)
-    {
-        TRACE ("Copying data back to screen");
-        XFixesSetPictureClipRegion (dpy, screen_info->rootBuffer, 0, 0, None);
-        XRenderComposite (dpy, PictOpSrc, screen_info->rootBuffer, None, screen_info->rootPicture,
-                          0, 0, 0, 0, 0, 0, screen_width, screen_height);
-    }
+    XFixesSetPictureClipRegion (dpy, screen_info->rootBuffer, 0, 0, None);
+    XRenderComposite (dpy, PictOpSrc, screen_info->rootBuffer, None, screen_info->rootPicture,
+                      0, 0, 0, 0, 0, 0, screen_width, screen_height);
 }
 
 static void
@@ -1257,8 +1254,26 @@ repair_win (CWindow *cw)
 
     if (parts)
     {
-        cw->damaged = TRUE;
+        GList *index;
+
+        /* Exclude opaque windows in front of this window from damage */
+        for (index = screen_info->cwindows; index; index = g_list_next (index))
+        {
+            CWindow *cw2 = (CWindow *) index->data;
+            if (cw2 == cw)
+            {
+                break;
+            }
+            else if ((cw2->opacity == NET_WM_OPAQUE) && (cw2->mode != WINDOW_ARGB) && 
+                     (cw2->viewable) && (cw2->damage) && (cw2->borderSize))
+            {
+                XFixesSubtractRegion (myScreenGetXDisplay (screen_info), parts,
+                                     parts, cw2->borderSize);
+            }
+        }
+
         add_damage (cw->screen_info, parts);
+        cw->damaged = TRUE;
     }
 }
 
@@ -1361,15 +1376,11 @@ set_win_opacity (CWindow *cw, guint opacity)
 static void
 map_win (CWindow *cw)
 {
-    ScreenInfo *screen_info = NULL;
-
     TRACE ("entering map_win");
     g_return_if_fail (cw != NULL);
 
     cw->viewable = TRUE;
     cw->damaged = FALSE;
-    screen_info = cw->screen_info;
-    add_repair (cw->screen_info->display_info);
 }
 
 static void
@@ -1550,37 +1561,21 @@ restack_win (CWindow *cw, Window above)
 void
 resize_win (CWindow *cw, gint x, gint y, gint width, gint height, gint bw, gboolean shape_notify)
 {
-    XserverRegion damage = None;
+    XserverRegion extents;
+    XserverRegion damage;
 
     g_return_if_fail (cw != NULL);
     TRACE ("entering resize_win");
+    TRACE ("resizing 0x%lx, (%i,%i) %ix%i", cw->id, x, y, width, height);
 
-    if (!(shape_notify) && (x == cw->attr.x) && (y == cw->attr.y) &&
-        (width == cw->attr.width) && (height == cw->attr.height) &&
-        (bw == cw->attr.border_width))
-    {
-        return;
-    }
-    
-    damage = XFixesCreateRegion (myScreenGetXDisplay (cw->screen_info), 0, 0);
-    if (cw->extents != None)	
+    damage = XFixesCreateRegion (myScreenGetXDisplay (cw->screen_info), NULL, 0);
+    if (cw->extents)    
     {
         XFixesCopyRegion (myScreenGetXDisplay (cw->screen_info), damage, cw->extents);
-    }
-
-    TRACE ("resizing 0x%lx, (%i,%i) %ix%i", cw->id, x, y, width, height);
-    if (cw->extents)
-    {
         XFixesDestroyRegion (myScreenGetXDisplay (cw->screen_info), cw->extents);
         cw->extents = None;
     }
 
-    if (cw->borderSize != None)
-    {
-        XFixesDestroyRegion (myScreenGetXDisplay (cw->screen_info), cw->borderSize);
-        cw->borderSize = None;
-    }
-
     if ((cw->attr.width != width) || (cw->attr.height != height))
     {
 #if HAVE_NAME_WINDOW_PIXMAP
@@ -1601,18 +1596,15 @@ resize_win (CWindow *cw, gint x, gint y, gint width, gint height, gint bw, gbool
             XRenderFreePicture (myScreenGetXDisplay (cw->screen_info), cw->shadow);
             cw->shadow = None;
         }
+    }
 
-        if (cw->borderSize != None)
+    if ((cw->attr.width != width) || (cw->attr.height != height) || (cw->attr.x != x) || (cw->attr.y != y))
+    {
+        if (cw->borderSize)
         {
             XFixesDestroyRegion (myScreenGetXDisplay (cw->screen_info), cw->borderSize);
             cw->borderSize = None;
         }
-
-        if (cw->extents)
-        {
-            XFixesDestroyRegion (myScreenGetXDisplay (cw->screen_info), cw->extents);
-            cw->extents = None;
-        }
     }
 
     cw->attr.x = x;
@@ -1621,13 +1613,10 @@ resize_win (CWindow *cw, gint x, gint y, gint width, gint height, gint bw, gbool
     cw->attr.height = height;
     cw->attr.border_width = bw;
 
-    if (damage)
-    {
-        XserverRegion extents = win_extents (cw);
-        XFixesUnionRegion (myScreenGetXDisplay (cw->screen_info), damage, damage, extents);
-        XFixesDestroyRegion (myScreenGetXDisplay (cw->screen_info), extents);
-        add_damage (cw->screen_info, damage);
-    }
+    extents = win_extents (cw);
+    XFixesUnionRegion (myScreenGetXDisplay (cw->screen_info), damage, damage, extents);
+    XFixesDestroyRegion (myScreenGetXDisplay (cw->screen_info), extents);
+    add_damage (cw->screen_info, damage);
 }
 
 static void
@@ -2140,6 +2129,9 @@ void
 compositorInitDisplay (DisplayInfo *display_info)
 {
 #ifdef HAVE_COMPOSITOR
+#if HAVE_NAME_WINDOW_PIXMAP
+    int composite_major, composite_minor;
+#endif
 
     g_return_if_fail (display_info != NULL);
     TRACE ("entering compositorInitDisplay");
@@ -2194,6 +2186,14 @@ compositorInitDisplay (DisplayInfo *display_info)
     {
         g_warning ("Compositing manager disabled.");
     }
+
+#if HAVE_NAME_WINDOW_PIXMAP
+    XCompositeQueryVersion (display_info->dpy, &composite_major, &composite_minor);
+    display_info->have_name_window_pixmap = ((composite_major > 0) || (composite_minor >= 2));
+#else
+    display_info->have_name_window_pixmap = FALSE;
+#endif
+
 #else /* HAVE_COMPOSITOR */
     display_info->enable_compositor = FALSE;
 #endif /* HAVE_COMPOSITOR */
