@@ -1,4 +1,4 @@
-/*      $Id: events.c,v 1.297.2.2 2005/02/12 21:29:37 olivier Exp $
+/*      $Id: events.c 24047 2006-12-04 20:32:53Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -205,6 +205,7 @@ typeOfClick (ScreenInfo *screen_info, Window w, XEvent * ev, gboolean allow_doub
     {
         TRACE ("grab failed in typeOfClick");
         gdk_beep ();
+        myScreenUngrabPointer (screen_info, ev->xbutton.time);
         return XFWM_BUTTON_UNDEFINED;
     }
 
@@ -297,15 +298,17 @@ moveRequest (Client * c, XEvent * ev)
 static void
 resizeRequest (Client * c, int corner, XEvent * ev)
 {
-    if (FLAG_TEST_ALL (c->xfwm_flags,
-            XFWM_FLAG_HAS_RESIZE | XFWM_FLAG_IS_RESIZABLE))
+    if (!FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN))
     {
-        clientResize (c, corner, ev);
-    }
-    else if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_MOVE)
-        && !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN))
-    {
-        clientMove (c, ev);
+        if (FLAG_TEST_ALL (c->xfwm_flags,
+                XFWM_FLAG_HAS_RESIZE | XFWM_FLAG_IS_RESIZABLE))
+        {
+            clientResize (c, corner, ev);
+        }
+        else if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_MOVE))
+        {
+            clientMove (c, ev);
+        }
     }
 }
 
@@ -481,7 +484,7 @@ handleKeyPress (DisplayInfo *display_info, XKeyEvent * ev)
                 clientClose (c);
                 break;
             case KEY_HIDE_WINDOW:
-                if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_BORDER) && CLIENT_CAN_HIDE_WINDOW (c))
+                if (CLIENT_CAN_HIDE_WINDOW (c))
                 {
                     clientHide (c, c->win_workspace, TRUE);
                 }
@@ -499,7 +502,7 @@ handleKeyPress (DisplayInfo *display_info, XKeyEvent * ev)
                 clientToggleShaded (c);
                 break;
             case KEY_STICK_WINDOW:
-                if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_BORDER) && CLIENT_CAN_STICK_WINDOW(c))
+                if (CLIENT_CAN_STICK_WINDOW(c))
                 {
                     clientToggleSticky (c, TRUE);
                     frameDraw (c, FALSE, FALSE);
@@ -541,8 +544,11 @@ handleKeyPress (DisplayInfo *display_info, XKeyEvent * ev)
             case KEY_MOVE_WORKSPACE_7:
             case KEY_MOVE_WORKSPACE_8:
             case KEY_MOVE_WORKSPACE_9:
-                clientRaise (c);
-                workspaceSwitch (screen_info, key - KEY_MOVE_WORKSPACE_1, c);
+                if (key - KEY_MOVE_WORKSPACE_1 < screen_info->workspace_count)
+                {
+                    clientRaise (c);
+                    workspaceSwitch (screen_info, key - KEY_MOVE_WORKSPACE_1, c);
+                }
                 break;
             default:
                 break;
@@ -608,7 +614,10 @@ handleKeyPress (DisplayInfo *display_info, XKeyEvent * ev)
         case KEY_WORKSPACE_7:
         case KEY_WORKSPACE_8:
         case KEY_WORKSPACE_9:
-            workspaceSwitch (screen_info, key - KEY_WORKSPACE_1, NULL);
+            if (key - KEY_WORKSPACE_1 < screen_info->workspace_count)
+            {
+                workspaceSwitch (screen_info, key - KEY_WORKSPACE_1, NULL);
+            }
             break;
         case KEY_SHORTCUT_1:
         case KEY_SHORTCUT_2:
@@ -1240,6 +1249,10 @@ handleConfigureNotify (DisplayInfo *display_info, XConfigureEvent * ev)
         screen_info->xscreen->width   = ev->width;
         screen_info->xscreen->height  = ev->height;
 #endif
+        setNetWorkarea (display_info, screen_info->xroot, screen_info->workspace_count, 
+                        gdk_screen_get_width (screen_info->gscr),
+                        gdk_screen_get_height (screen_info->gscr),
+                        screen_info->margins);
         placeSidewalks (screen_info, screen_info->params->wrap_workspaces);
         clientScreenResize (screen_info);
     }
@@ -1586,6 +1599,7 @@ handleFocusOut (DisplayInfo *display_info, XFocusChangeEvent * ev)
         if ((c) && (c == clientGetFocus ()))
         {
             TRACE ("focus lost from \"%s\" (0x%lx)", c->name, c->window);
+            clientPassGrabMouseButton (NULL);
             clientUpdateFocus (c->screen_info, NULL, NO_FOCUS_FLAG);
             /* Clear timeout */
             clear_timeout ();
@@ -1598,9 +1612,6 @@ handlePropertyNotify (DisplayInfo *display_info, XPropertyEvent * ev)
 {
     ScreenInfo *screen_info = NULL;
     Client *c = NULL;
-    Window w = None;
-    char *names;
-    int length;
 
     TRACE ("entering handlePropertyNotify");
 
@@ -1648,12 +1659,32 @@ handlePropertyNotify (DisplayInfo *display_info, XPropertyEvent * ev)
         }
         else if (ev->atom == display_info->atoms[WM_TRANSIENT_FOR])
         {
+            Window w = None;
+
             TRACE ("client \"%s\" (0x%lx) has received a wm_transient_for notify", c->name, c->window);
             getTransientFor (display_info, c->screen_info->xroot, c->window, &w);
             if (clientCheckTransientWindow (c, w))
             {
                 c->transient_for = w;
+#if 0                
+                /*
+                  Java 1.6 updates the WM_TRANSIENT_FOR properties "on-the-fly"
+                  of its windows to maintain the z-order. 
+                  
+                  If we raise the transient then, we clearly have a race 
+                  condition between the WM and Java... And that breaks 
+                  the z-order. Bug #2483.
+                  
+                  I still think that raising here makes sense, to ensure
+                  that the newly promoted transient window is placed above
+                  its parent.
+                  
+                  Chances are that Java 1.6 won't change any time soon (heh,
+                  it's not even released yet), so let's adjust the WM to
+                  work with Java 1.6...
+                 */
                 clientRaise (c);
+#endif
             }
         }
         else if (ev->atom == display_info->atoms[WIN_HINTS])
@@ -1721,10 +1752,13 @@ handlePropertyNotify (DisplayInfo *display_info, XPropertyEvent * ev)
 
     if (ev->atom == display_info->atoms[NET_DESKTOP_NAMES])
     {
+        char **names;
+        int items;
+
         TRACE ("root has received a net_desktop_names notify");
-        if (getUTF8String (display_info, screen_info->xroot, NET_DESKTOP_NAMES, &names, &length))
+        if (getUTF8StringList (display_info, screen_info->xroot, NET_DESKTOP_NAMES, &names, &items))
         {
-            workspaceSetNames (screen_info, names, length);
+            workspaceSetNames (screen_info, names, items);
         }
     }
     else if (ev->atom == display_info->atoms[GNOME_PANEL_DESKTOP_AREA])
@@ -1737,6 +1771,7 @@ handlePropertyNotify (DisplayInfo *display_info, XPropertyEvent * ev)
     {
         TRACE ("root has received a net_desktop_layout notify");
         getDesktopLayout(display_info, screen_info->xroot, screen_info->workspace_count, &screen_info->desktop_layout);
+        placeSidewalks(screen_info, screen_info->params->wrap_workspaces);
     }
 }
 
@@ -1830,15 +1865,31 @@ handleClientMessage (DisplayInfo *display_info, XClientMessageEvent * ev)
         else if ((ev->message_type == display_info->atoms[NET_ACTIVE_WINDOW]) && (ev->format == 32))
         {
             TRACE ("client \"%s\" (0x%lx) has received a net_active_window event", c->name, c->window);
-            clientSetWorkspace (c, screen_info->current_ws, TRUE);
-            clientShow (c, TRUE);
-            clientRaise (c);
             if (ev->data.l[0] != 0)
             {
-                clientSetFocus (screen_info, c, (Time) ev->data.l[1], NO_FOCUS_FLAG);
+                Time current = myDisplayGetCurrentTime (screen_info->display_info);
+                Time ev_time = (Time) ev->data.l[1];
+
+                /* We are simply ignoring XServer time wraparound here */
+                TRACE ("Time of event received is %u, current XServer time is %u", ev_time, current);
+                if ((ev_time != (Time) 0) && (ev_time < current))
+                {
+                    FLAG_SET (c->flags, CLIENT_FLAG_DEMANDS_ATTENTION);
+                    clientSetNetState (c);
+                }
+                else
+                {
+                    clientSetWorkspace (c, screen_info->current_ws, TRUE);
+                    clientShow (c, TRUE);
+                    clientRaise (c);
+                    clientSetFocus (screen_info, c, (Time) ev_time, NO_FOCUS_FLAG);
+                }
             }
             else
             {
+                clientSetWorkspace (c, screen_info->current_ws, TRUE);
+                clientShow (c, TRUE);
+                clientRaise (c);
                 clientSetFocus (screen_info, c, CurrentTime, NO_FOCUS_FLAG);
             }
         }
@@ -1860,8 +1911,8 @@ handleClientMessage (DisplayInfo *display_info, XClientMessageEvent * ev)
         if (((ev->message_type == display_info->atoms[WIN_WORKSPACE]) || 
              (ev->message_type == display_info->atoms[NET_CURRENT_DESKTOP])) && (ev->format == 32))
         {
-            TRACE ("root has received a win_workspace or a net_current_desktop event");
-            if (ev->data.l[0] != screen_info->current_ws)
+            TRACE ("root has received a win_workspace or a net_current_desktop event %i", ev->data.l[0]);
+            if ((ev->data.l[0] >= 0) && (ev->data.l[0] < screen_info->workspace_count) && (ev->data.l[0] != screen_info->current_ws))
             {
                 workspaceSwitch (screen_info, ev->data.l[0], NULL);
             }
@@ -1939,6 +1990,29 @@ handleColormapNotify (DisplayInfo *display_info, XColormapEvent * ev)
     }
 }
 
+static void
+handleMappingNotify (DisplayInfo *display_info, XMappingEvent * ev)
+{
+    TRACE ("entering handleMappingNotify");
+
+    /* Refreshes the stored modifier and keymap information */
+    XRefreshKeyboardMapping (ev);
+
+    /* Update internal modifiers masks if necessary */
+    if (ev->request == MappingModifier)
+    {
+        TRACE ("handleMappingNotify: modifiers mapping has changed");
+        initModifiers (display_info->dpy);
+    }
+
+    /* Regrab all keys if the notify is for keyboard (ie not pointer) */
+    if (ev->request != MappingPointer)
+    {
+        TRACE ("handleMappingNotify: Reload settings");
+        reloadSettings (display_info, UPDATE_BUTTON_GRABS);
+    }
+}
+
 static void
 handleEvent (DisplayInfo *display_info, XEvent * ev)
 {
@@ -2002,8 +2076,11 @@ handleEvent (DisplayInfo *display_info, XEvent * ev)
         case ColormapNotify:
             handleColormapNotify (display_info, (XColormapEvent *) ev);
             break;
+        case MappingNotify:
+            handleMappingNotify (display_info, (XMappingEvent *) ev);
+            break;
         default:
-            if (display_info->shape && (ev->type == display_info->shape_event))
+            if ((display_info->shape) && (ev->type == display_info->shape_event))
             {
                 handleShape (display_info, (XShapeEvent *) ev);
             }
@@ -2286,7 +2363,7 @@ show_popup_cb (GtkWidget * widget, GdkEventButton * ev, gpointer data)
 
     menu = menu_default (screen_info->gscr, c->window, ops, insensitive, menu_callback, 
                          c->win_workspace, screen_info->workspace_count, 
-                         screen_info->workspace_names, screen_info->workspace_names_length,
+                         screen_info->workspace_names, screen_info->workspace_names_items,
                          display_info->xfilter, screen_info);
 
     if (!menu_popup (menu, x, y, ev->button, ev->time))
