@@ -1,4 +1,4 @@
-/*      $Id: focus.c,v 1.43.2.4 2005/02/19 14:13:30 olivier Exp $
+/*      $Id: focus.c 22892 2006-08-26 21:02:28Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -137,7 +137,7 @@ clientFocusNew(Client * c)
     }
     screen_info = c->screen_info;
     display_info = screen_info->display_info;
-    give_focus = screen_info->params->focus_new;
+    give_focus = (c-> type & WINDOW_REGULAR_FOCUSABLE) && (screen_info->params->focus_new);
     prevent_focus_stealing = screen_info->params->prevent_focus_stealing;
 
     /*  Try to avoid focus stealing */
@@ -146,7 +146,7 @@ clientFocusNew(Client * c)
         if (FLAG_TEST(c->flags, CLIENT_FLAG_HAS_USER_TIME) &&
             FLAG_TEST(client_focus->flags, CLIENT_FLAG_HAS_USER_TIME))
         {
-            TRACE ("Current %u, new %u", client_focus->user_time, c->user_time);
+            TRACE ("Current %u, new %u", (unsigned int) client_focus->user_time,(unsigned int)  c->user_time);
             if (c->user_time < client_focus->user_time)
             {
                 give_focus = FALSE;
@@ -157,7 +157,7 @@ clientFocusNew(Client * c)
     clientGrabMouseButton (c);
     if (give_focus || FLAG_TEST(c->flags, CLIENT_FLAG_STATE_MODAL))
     {
-        if (client_focus)
+        if ((client_focus) && !(clientIsTransientOrModalFor (c, client_focus)))
         {
             if (clientAdjustFullscreenLayer (client_focus, FALSE))
             {
@@ -165,14 +165,17 @@ clientFocusNew(Client * c)
             }
         }
         clientSetFocus (screen_info, c, 
+#if 0
                         myDisplayGetCurrentTime (display_info), 
+#else
+                        CurrentTime, 
+#endif
                         FOCUS_IGNORE_MODAL);
     }
     else
     {
         FLAG_SET (c->flags, CLIENT_FLAG_DEMANDS_ATTENTION);
         clientSetNetState (c);
-        clientGrabMouseButton (c);
     }
 }
 
@@ -209,6 +212,7 @@ clientSelectMask (Client * c, int mask, int type)
     {
         return TRUE;
     }
+
     return FALSE;
 }
 
@@ -283,6 +287,11 @@ clientPassFocus (ScreenInfo *screen_info, Client *c, Client *exclude)
         return;
     }
 
+    if (current_focus == last_ungrab)
+    {
+        clientPassGrabMouseButton (NULL);
+    }
+
     display_info = screen_info->display_info;
     top_most = clientGetTopMostFocusable (screen_info, look_in_layer, exclude);
     if (screen_info->params->click_to_focus)
@@ -296,7 +305,6 @@ clientPassFocus (ScreenInfo *screen_info, Client *c, Client *exclude)
                    for group, so it safe to use clientGetTransient because 
                    it's really what we want...
                  */
-
                 c2 = clientGetTransient (c);
                 if (c2 && FLAG_TEST(c2->xfwm_flags, XFWM_FLAG_VISIBLE))
                 {
@@ -308,10 +316,7 @@ clientPassFocus (ScreenInfo *screen_info, Client *c, Client *exclude)
             else
             {
                 c2 = clientGetNext (c, 0);
-                /* Send focus back to the previous window only if it's on a different
-                   layer, to avoid sending focus back to a lowered window....
-                 */
-                if ((c2) && (c2->win_layer != c->win_layer))
+                if ((c2) && (c2->win_layer >= c->win_layer))
                 {
                     new_focus = c2;
                 }
@@ -327,7 +332,11 @@ clientPassFocus (ScreenInfo *screen_info, Client *c, Client *exclude)
         new_focus = top_most.prefered ? top_most.prefered : top_most.highest;
     }
     clientSetFocus (screen_info, new_focus, 
+#if 0
                     myDisplayGetCurrentTime (screen_info->display_info), 
+#else
+                    CurrentTime, 
+#endif
                     FOCUS_IGNORE_MODAL | FOCUS_FORCE);
 }
 
@@ -401,7 +410,11 @@ clientUpdateFocus (ScreenInfo *screen_info, Client * c, unsigned short flags)
         return;
     }
 
-    if (c == clientGetLastRaise (screen_info))
+    /* 
+       We can release the button mouse grab if we don't raise on click or if the focused window
+       is the one that has been raised at last.
+     */
+    if (!(screen_info->params->raise_on_click) || (c == clientGetLastRaise (screen_info)))
     {
         clientPassGrabMouseButton (c);
     }
@@ -547,14 +560,9 @@ clientUngrabMouseButton (Client * c)
     TRACE ("ungrabing buttons for client \"%s\" (0x%lx)", c->name, c->window);
 
     screen_info = c->screen_info;
-    if (screen_info->params->raise_with_any_button)
-    {
-        ungrabButton(clientGetXDisplay (c), AnyButton, 0, c->window);
-    }
-    else
-    {
-        ungrabButton(clientGetXDisplay (c), Button1, 0, c->window);
-    }
+    ungrabButton(clientGetXDisplay (c), AnyButton, AnyModifier, c->window);
+    /* We've ungrabbed way too much, so regrab the regular buttons/modifiers */
+    clientGrabButtons (c);
 }
 
 void
@@ -593,7 +601,6 @@ void
 clientPassGrabMouseButton (Client * c)
 {
     TRACE ("entering clientPassMouseGrabButton");
-    TRACE ("ungrabing buttons for client \"%s\" (0x%lx)", c->name, c->window);
 
     if (c == NULL)
     {
@@ -605,6 +612,7 @@ clientPassGrabMouseButton (Client * c)
         return;
     }
 
+    TRACE ("ungrabing buttons for client \"%s\" (0x%lx)", c->name, c->window);
     if (last_ungrab == c)
     {
         return;
