@@ -1,4 +1,4 @@
-/*      $Id: frame.c 16003 2005-06-16 19:18:08Z olivier $
+/*      $Id: frame.c 22260 2006-07-04 17:42:55Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -750,6 +750,11 @@ frameSetShape (Client * c, int state, ClientPixmapCache * pm_cache, int button_x
         }
     }
 
+    rect.x = 0;
+    rect.y = 0;
+    rect.width  = frameWidth (c);
+    rect.height = frameHeight (c);
+    XShapeCombineRectangles (display_info->dpy, temp, ShapeBounding, 0, 0, &rect, 1, ShapeIntersect, 0);
     XShapeCombineShape (display_info->dpy, c->frame, ShapeBounding, 0, 0, temp, ShapeBounding, ShapeSet);
 
     XDestroyWindow (display_info->dpy, temp);
