@@ -1,4 +1,4 @@
-/*      $Id: hints.c 16044 2005-06-22 21:56:39Z olivier $
+/*      $Id: hints.c 21734 2006-05-20 19:54:52Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -119,15 +119,17 @@ getMotifHints (DisplayInfo *display_info, Window w)
 unsigned int
 getWMProtocols (DisplayInfo *display_info, Window w)
 {
-    Atom *protocols = None, *ap;
+    Atom *protocols, *ap;
     int i, n;
     Atom atype;
     int aformat;
-    unsigned int result = 0;
+    unsigned int result;
     unsigned long bytes_remain, nitems;
+    unsigned char *data;
 
     TRACE ("entering getWMProtocols");
 
+    result = 0;
     if (XGetWMProtocols (display_info->dpy, w, &protocols, &n))
     {
         for (i = 0, ap = protocols; i < n; i++, ap++)
@@ -146,14 +148,20 @@ getWMProtocols (DisplayInfo *display_info, Window w)
                 result |= WM_PROTOCOLS_CONTEXT_HELP;
             }
         }
+        if (protocols)
+        {
+            XFree (protocols);
+        }
     }
     else
     {
-        if ((XGetWindowProperty (display_info->dpy, w, display_info->atoms[WM_PROTOCOLS], 0L, 10L, FALSE,
-                    display_info->atoms[WM_PROTOCOLS], &atype, &aformat, &nitems, &bytes_remain,
-                    (unsigned char **) &protocols)) == Success)
+        if ((XGetWindowProperty (display_info->dpy, w,
+                    display_info->atoms[WM_PROTOCOLS], 0L, 10L, FALSE,
+                    display_info->atoms[WM_PROTOCOLS], &atype,
+                    &aformat, &nitems, &bytes_remain,
+                    (unsigned char **) &data)) == Success)
         {
-            for (i = 0, ap = protocols; i < nitems; i++, ap++)
+            for (i = 0, ap = (Atom *) data; i < nitems; i++, ap++)
             {
                 if (*ap == display_info->atoms[WM_TAKE_FOCUS])
                 {
@@ -169,12 +177,12 @@ getWMProtocols (DisplayInfo *display_info, Window w)
                     result |= WM_PROTOCOLS_CONTEXT_HELP;
                 }
             }
+            if (data)
+            {
+                XFree (data);
+            }
         }
     }
-    if (protocols)
-    {
-        XFree (protocols);
-    }
     return result;
 }
 
@@ -567,61 +575,21 @@ getTransientFor (DisplayInfo *display_info, Window root, Window w, Window * tran
     TRACE ("Window (0x%lx) is transient for (0x%lx)", w, *transient_for);
 }
 
-gboolean
-getUTF8String (DisplayInfo *display_info, Window w, int atom_id, char **str_p, int *length)
+static gchar *
+internal_utf8_strndup (const gchar *src, gssize max_len)
 {
-    Atom type;
-    int format;
-    unsigned long bytes_after;
-    unsigned char *str;
-    unsigned long n_items;
+    const gchar *s = src;
 
-    g_return_val_if_fail (((atom_id > 0) && (atom_id < NB_ATOMS)), FALSE);
-    TRACE ("entering getUTF8String");
+    if (max_len <= 0)
+	return g_strdup (src);
 
-    *str_p = NULL;
-    if ((XGetWindowProperty (display_info->dpy, w, display_info->atoms[atom_id], 
-                             0, G_MAXLONG, FALSE, display_info->atoms[UTF8_STRING], &type, 
-                             &format, &n_items, &bytes_after, (unsigned char **) &str) != Success) || (type == None))
+    while (max_len && *s)
     {
-        TRACE ("no utf8_string value provided");
-        return FALSE;
+	s = g_utf8_next_char (s);
+	max_len--;
     }
 
-    if (!check_type_and_format (8, display_info->atoms[UTF8_STRING], -1, format, type))
-    {
-        TRACE ("utf8_string value invalid");
-        if (str)
-        {
-            XFree (str);
-        }
-        return FALSE;
-    }
-
-    if (!g_utf8_validate (str, -1, NULL))
-    {
-        char *name;
-
-        name = XGetAtomName (display_info->dpy, display_info->atoms[atom_id]);
-        if (name)
-        {
-            TRACE ("Property %s on window 0x%lx contains invalid UTF-8 characters", name, w);
-            XFree (name);
-        }
-        XFree (str);
-
-        return FALSE;
-    }
-
-    if (str)
-    {
-        xfce_utf8_remove_controls((gchar *) str, -1, NULL);
-    }
-    
-    *str_p = str;
-    *length = n_items;
-
-    return TRUE;
+    return g_strndup (src, s - src);
 }
 
 static char *
@@ -661,7 +629,7 @@ get_text_property (DisplayInfo *display_info, Window w, Atom a)
         retval = text_property_to_utf8 (display_info, &text);
         if (retval)
         {
-            xfce_utf8_remove_controls((gchar *) retval, -1, NULL);
+            xfce_utf8_remove_controls((gchar *) retval, MAX_STR_LENGTH, NULL);
         }
         if ((text.value) && (text.nitems > 0))
         {
@@ -677,6 +645,137 @@ get_text_property (DisplayInfo *display_info, Window w, Atom a)
     return retval;
 }
 
+static gboolean
+getUTF8StringData (DisplayInfo *display_info, Window w, int atom_id, char **str_p, int *length)
+{
+    Atom type;
+    int format;
+    unsigned long bytes_after;
+    unsigned char *str;
+    unsigned long n_items;
+
+    g_return_val_if_fail (((atom_id > 0) && (atom_id < NB_ATOMS)), FALSE);
+    TRACE ("entering getUTF8StringData");
+
+    *str_p = NULL;
+    if ((XGetWindowProperty (display_info->dpy, w, display_info->atoms[atom_id], 
+                             0, G_MAXLONG, FALSE, display_info->atoms[UTF8_STRING], &type, 
+                             &format, &n_items, &bytes_after, (unsigned char **) &str) != Success) || (type == None))
+    {
+        TRACE ("no UTF8_STRING property found");
+        return FALSE;
+    }
+
+    if (!check_type_and_format (8, display_info->atoms[UTF8_STRING], -1, format, type))
+    {
+        TRACE ("UTF8_STRING value invalid");
+        if (str)
+        {
+            XFree (str);
+        }
+        return FALSE;
+    }
+
+    *str_p = (char *) str;
+    *length = n_items;
+
+    return TRUE;
+}
+
+gboolean
+getUTF8String (DisplayInfo *display_info, Window w, int atom_id, char **str_p, int *length)
+{
+    char *xstr;
+    
+    g_return_val_if_fail (((atom_id > 0) && (atom_id < NB_ATOMS)), FALSE);
+    TRACE ("entering getUTF8String");
+
+    if (!getUTF8StringData (display_info, w, atom_id, &xstr, length))
+    {
+        *str_p = NULL;
+        *length = 0;
+
+        return FALSE;
+    }
+
+    /* gmalloc the returned string */
+    *str_p = internal_utf8_strndup (xstr, MAX_STR_LENGTH);
+    XFree (xstr);
+    
+    if (!g_utf8_validate (*str_p, -1, NULL))
+    {
+        TRACE ("getUTF8String() returned invalid UTF-8 characters");
+        g_free (*str_p);
+        str_p = NULL;
+        length = 0;
+
+        return FALSE;
+    }
+
+    if (*str_p)
+    {
+        xfce_utf8_remove_controls((gchar *) *str_p, -1, NULL);
+    }
+    
+    return TRUE;
+}
+
+gboolean
+getUTF8StringList (DisplayInfo *display_info, Window w, int atom_id, char ***str_p, int *n_items)
+{
+    char *xstr, *ptr;
+    char **retval;
+    guint i;
+    int length;
+
+    g_return_val_if_fail (((atom_id > 0) && (atom_id < NB_ATOMS)), FALSE);
+
+    TRACE ("entering getUTF8StringList");
+
+    *str_p = NULL;
+    *n_items = 0;
+
+    if (!getUTF8StringData (display_info, w, atom_id, &xstr, &length) || !length)
+    {
+        return FALSE;
+    }
+    
+    i = 0;
+    while (i < length)
+    {
+        if (!xstr[i])
+        {
+            *n_items = *n_items + 1;
+        }
+        i++;
+    }
+    if (xstr[length - 1])
+    {
+        *n_items = *n_items + 1;
+    }
+ 
+    retval = g_new0 (char *, *n_items + 1);
+    ptr = xstr;
+    
+    for (i = 0; i < *n_items; i++)
+    {
+        if (g_utf8_validate (ptr, -1, NULL))
+        {
+            retval[i] = internal_utf8_strndup (ptr, MAX_STR_LENGTH);
+        }
+        else
+        {
+            TRACE ("getUTF8StringList() returned invalid UTF-8 characters in string #%i.", i);
+            retval[i] = g_strdup ("");
+        }
+        ptr += strlen (ptr) + 1;
+    }
+    XFree (xstr);
+    *str_p = retval;    
+
+    return TRUE;
+}
+
 void
 getWindowName (DisplayInfo *display_info, Window w, char **name)
 {
@@ -689,16 +788,16 @@ getWindowName (DisplayInfo *display_info, Window w, char **name)
     *name = NULL;
     g_return_if_fail (w != None);
 
-    if (getUTF8String (display_info, w, NET_WM_NAME, &str, &len))
+    if (getUTF8StringData (display_info, w, NET_WM_NAME, &str, &len))
     {
-        *name = strdup (str);
+        *name = internal_utf8_strndup (str, MAX_STR_LENGTH);
         XFree (str);
         return;
     }
     str = get_text_property (display_info, w, XA_WM_NAME);
     if (str)
     {
-        *name = strdup (str);
+        *name = internal_utf8_strndup (str, MAX_STR_LENGTH);
         XFree (str);
     }
     else
@@ -1004,7 +1103,7 @@ getWindowStartupId (DisplayInfo *display_info, Window w, char **startup_id)
     *startup_id = NULL;
     g_return_val_if_fail (w != None, FALSE);
 
-    if (getUTF8String (display_info, w, NET_STARTUP_ID, &str, &len))
+    if (getUTF8StringData (display_info, w, NET_STARTUP_ID, &str, &len))
     {
         *startup_id = strdup (str);
         XFree (str);
