@@ -1,4 +1,4 @@
-/*      $Id: hints.h 12159 2004-11-23 23:57:33Z olivier $
+/*      $Id: hints.h 20529 2006-03-24 14:46:19Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -37,6 +37,7 @@
 #include "display.h"
 
 #define MWM_HINTS_ELEMENTS                      3L
+#define MAX_STR_LENGTH                          255
 
 #define MWM_HINTS_FUNCTIONS                     (1L << 0)
 #define MWM_HINTS_DECORATIONS                   (1L << 1)
@@ -159,6 +160,7 @@ void setUTF8StringHint (DisplayInfo *, Window, int, const char *);
 void getTransientFor (DisplayInfo *, Window, Window, Window *);
 void getWindowName (DisplayInfo *, Window, char **);
 gboolean getUTF8String (DisplayInfo *, Window, int, char **, int *);
+gboolean getUTF8StringList (DisplayInfo *, Window, int, char ***, int *);
 void getWindowName (DisplayInfo *, Window, char **);
 gboolean checkKdeSystrayWindow(DisplayInfo *, Window);
 void sendSystrayReqDock(DisplayInfo *, Window, Window);
