@@ -1,4 +1,4 @@
-/*      $Id: netwm.c 18616 2005-11-03 21:42:19Z olivier $
+/*      $Id: netwm.c 20325 2006-03-10 23:36:36Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -925,6 +925,8 @@ clientWindowType (Client * c)
             TRACE ("atom net_wm_window_type_dock detected");
             c->type = WINDOW_DOCK;
             c->initial_layer = WIN_LAYER_DOCK;
+            FLAG_SET (c->flags,
+                CLIENT_FLAG_SKIP_PAGER | CLIENT_FLAG_SKIP_TASKBAR);
             FLAG_UNSET (c->xfwm_flags,
                 XFWM_FLAG_HAS_BORDER |  XFWM_FLAG_HAS_MOVE |
                 XFWM_FLAG_HAS_HIDE | XFWM_FLAG_HAS_MAXIMIZE | 
@@ -935,6 +937,8 @@ clientWindowType (Client * c)
             TRACE ("atom net_wm_window_type_toolbar detected");
             c->type = WINDOW_TOOLBAR;
             c->initial_layer = WIN_LAYER_NORMAL;
+            FLAG_SET (c->flags,
+                CLIENT_FLAG_SKIP_PAGER | CLIENT_FLAG_SKIP_TASKBAR);
             FLAG_UNSET (c->xfwm_flags,
                 XFWM_FLAG_HAS_HIDE | XFWM_FLAG_HAS_MAXIMIZE |
                 XFWM_FLAG_HAS_STICK);
@@ -969,6 +973,8 @@ clientWindowType (Client * c)
         else if (c->type_atom == display_info->atoms[NET_WM_WINDOW_TYPE_UTILITY])
         {
             TRACE ("atom net_wm_window_type_utility detected");
+            FLAG_SET (c->flags,
+                CLIENT_FLAG_SKIP_PAGER | CLIENT_FLAG_SKIP_TASKBAR);
             c->type = WINDOW_UTILITY;
             c->initial_layer = WIN_LAYER_NORMAL;
         }
@@ -977,6 +983,8 @@ clientWindowType (Client * c)
             TRACE ("atom net_wm_window_type_splash detected");
             c->type = WINDOW_SPLASHSCREEN;
             c->initial_layer = WIN_LAYER_ABOVE_DOCK;
+            FLAG_SET (c->flags,
+                CLIENT_FLAG_SKIP_PAGER | CLIENT_FLAG_SKIP_TASKBAR);
             FLAG_UNSET (c->xfwm_flags,
                 XFWM_FLAG_HAS_BORDER | XFWM_FLAG_HAS_HIDE |
                 XFWM_FLAG_HAS_MENU | XFWM_FLAG_HAS_MOVE |
@@ -997,16 +1005,9 @@ clientWindowType (Client * c)
         TRACE ("Window \"%s\" is a transient or a modal", c->name);
 
         c2 = clientGetHighestTransientOrModalFor (c);
-        if ((c2) && (c->win_layer < c2->win_layer))
+        if ((c2) && (c->initial_layer < c2->win_layer))
         {
-            if (clientIsTransient (c))
-            {
-                c->initial_layer = c2->win_layer;
-            }
-            else if (c->initial_layer < c2->win_layer) /* clientIsModal (c) */
-            {
-                c->initial_layer = c2->win_layer;
-            }
+            c->initial_layer = c2->win_layer;
             TRACE ("Applied layer is %i", c->initial_layer);
         }
         FLAG_UNSET (c->xfwm_flags, XFWM_FLAG_HAS_HIDE | XFWM_FLAG_HAS_STICK);
