@@ -1,4 +1,4 @@
-/*      $Id: placement.c 12401 2005-03-04 22:28:16Z olivier $
+/*      $Id: placement.c 19276 2006-01-02 21:38:07Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -123,6 +123,13 @@ clientAutoMaximize (Client * c)
     gint monitor_nbr;
     int cx, cy;
 
+    if (FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN) || 
+        FLAG_TEST (c->xfwm_flags, XFWM_FLAG_LEGACY_FULLSCREEN))
+    {
+        /* Fullscree nwindows should not be maximized */
+        return;
+    }
+
     cx = frameX (c) + (frameWidth (c) / 2);
     cy = frameY (c) + (frameHeight (c) / 2);
 
