@@ -1,4 +1,4 @@
-/*      $Id: screen.c 12431 2005-03-22 22:37:42Z olivier $
+/*      $Id: screen.c 20529 2006-03-24 14:46:19Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -106,7 +106,7 @@ myScreenInit (DisplayInfo *display_info, GdkScreen *gscr, unsigned long event_ma
 
     screen_info->workspace_count = -1;
     screen_info->workspace_names = NULL;
-    screen_info->workspace_names_length = 0;
+    screen_info->workspace_names_items = 0;
 
     screen_info->mcs_client = NULL;
     screen_info->mcs_initted = FALSE;
@@ -199,9 +199,12 @@ myScreenClose (ScreenInfo *screen_info)
     compositorUnmanageScreen (screen_info);
     closeSettings (screen_info);
 
-    g_free (screen_info->workspace_names);
+    if (screen_info->workspace_names)
+    {
+        g_strfreev (screen_info->workspace_names);
+    }
     screen_info->workspace_names = NULL;
-    screen_info->workspace_names_length = 0;
+    screen_info->workspace_names_items = 0;
 
     xfwmWindowDelete (&screen_info->sidewalk[0]);
     xfwmWindowDelete (&screen_info->sidewalk[1]);
@@ -339,3 +342,43 @@ myScreenUngrabPointer (ScreenInfo *screen_info, Time time)
 
     return screen_info->pointer_grabs;
 }
+
+void
+myScreenGrabKeys (ScreenInfo *screen_info)
+{
+    Display *dpy;
+
+    dpy = myScreenGetXDisplay (screen_info);
+
+    ungrabKeys (dpy, screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_CYCLE_WINDOWS], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_NEXT_WORKSPACE], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_PREV_WORKSPACE], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_ADD_WORKSPACE], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_DEL_WORKSPACE], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_WORKSPACE_1], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_WORKSPACE_2], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_WORKSPACE_3], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_WORKSPACE_4], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_WORKSPACE_5], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_WORKSPACE_6], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_WORKSPACE_7], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_WORKSPACE_8], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_WORKSPACE_9], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_SHORTCUT_1], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_SHORTCUT_2], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_SHORTCUT_3], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_SHORTCUT_4], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_SHORTCUT_5], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_SHORTCUT_6], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_SHORTCUT_7], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_SHORTCUT_8], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_SHORTCUT_9], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_SHORTCUT_10], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_UP_WORKSPACE], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_DOWN_WORKSPACE], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_LEFT_WORKSPACE], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_RIGHT_WORKSPACE], screen_info->xroot);
+}
+
+
