@@ -1,4 +1,4 @@
-/*      $Id: settings.c,v 1.122.2.1 2005/02/04 21:25:44 olivier Exp $
+/*      $Id: settings.c 20529 2006-03-24 14:46:19Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -68,15 +68,15 @@ static void              watch_cb             (Window,
                                                long, 
                                                void *);
 static void              loadRcData           (ScreenInfo *, 
-                                               Settings rc[]);
+                                               Settings *);
 static void              loadMcsData          (ScreenInfo *,
-                                               Settings rc[]);
+                                               Settings *);
 static void              loadTheme            (ScreenInfo *, 
-                                               Settings rc[]);
+                                               Settings *);
 static void              loadShortcutCmd      (ScreenInfo *, 
-                                               Settings rc[]);
+                                               Settings *);
 static gboolean          loadKeyBindings      (ScreenInfo *, 
-                                               Settings rc[]);
+                                               Settings *);
 static void              unloadTheme          (ScreenInfo *);
 static void              unloadSettings       (ScreenInfo *);
 static gboolean          reloadScreenSettings (ScreenInfo *, 
@@ -344,7 +344,7 @@ notify_cb (const char *name, const char *channel_name, McsAction action, McsSett
                 {
                     if (!strcmp (name, "Xfwm/KeyThemeName"))
                     {
-                        reloadScreenSettings (screen_info, UPDATE_KEYGRABS);
+                        reloadScreenSettings (screen_info, UPDATE_KEY_GRABS);
                     }
                 }
                 break;
@@ -398,7 +398,7 @@ watch_cb (Window window, Bool is_start, long mask, void *cb_data)
 }
 
 static void
-loadRcData (ScreenInfo *screen_info, Settings rc[])
+loadRcData (ScreenInfo *screen_info, Settings *rc)
 {
     gchar *homedir;
     gchar *keythemevalue;
@@ -434,7 +434,7 @@ loadRcData (ScreenInfo *screen_info, Settings rc[])
 }
 
 static void
-loadMcsData (ScreenInfo *screen_info, Settings rc[])
+loadMcsData (ScreenInfo *screen_info, Settings *rc)
 {
     McsSetting *setting;
     if (screen_info->mcs_client)
@@ -621,7 +621,7 @@ loadMcsData (ScreenInfo *screen_info, Settings rc[])
 }
 
 static void
-loadTheme (ScreenInfo *screen_info, Settings rc[])
+loadTheme (ScreenInfo *screen_info, Settings *rc)
 {
     GtkWidget *widget = NULL;
     DisplayInfo *display_info = NULL;
@@ -916,7 +916,7 @@ loadTheme (ScreenInfo *screen_info, Settings rc[])
 }
 
 static void
-loadShortcutCmd (ScreenInfo *screen_info, Settings rc[])
+loadShortcutCmd (ScreenInfo *screen_info, Settings *rc)
 {
     int i;
 
@@ -939,7 +939,7 @@ loadShortcutCmd (ScreenInfo *screen_info, Settings rc[])
 }
 
 static gboolean
-loadKeyBindings (ScreenInfo *screen_info, Settings rc[])
+loadKeyBindings (ScreenInfo *screen_info, Settings *rc)
 {
     Display *dpy;
     gchar *keytheme;
@@ -1035,37 +1035,7 @@ loadKeyBindings (ScreenInfo *screen_info, Settings rc[])
     parseKeyString (dpy, &screen_info->params->keys[KEY_MOVE_DOWN_WORKSPACE], getValue ("move_window_down_workspace_key", rc));
     parseKeyString (dpy, &screen_info->params->keys[KEY_MOVE_LEFT_WORKSPACE], getValue ("move_window_left_workspace_key", rc));
     parseKeyString (dpy, &screen_info->params->keys[KEY_MOVE_RIGHT_WORKSPACE], getValue ("move_window_right_workspace_key", rc));
-
-    ungrabKeys (dpy, screen_info->gnome_win);
-    grabKey (dpy, &screen_info->params->keys[KEY_CYCLE_WINDOWS], screen_info->gnome_win);
-    grabKey (dpy, &screen_info->params->keys[KEY_NEXT_WORKSPACE], screen_info->gnome_win);
-    grabKey (dpy, &screen_info->params->keys[KEY_PREV_WORKSPACE], screen_info->gnome_win);
-    grabKey (dpy, &screen_info->params->keys[KEY_ADD_WORKSPACE], screen_info->gnome_win);
-    grabKey (dpy, &screen_info->params->keys[KEY_DEL_WORKSPACE], screen_info->gnome_win);
-    grabKey (dpy, &screen_info->params->keys[KEY_WORKSPACE_1], screen_info->gnome_win);
-    grabKey (dpy, &screen_info->params->keys[KEY_WORKSPACE_2], screen_info->gnome_win);
-    grabKey (dpy, &screen_info->params->keys[KEY_WORKSPACE_3], screen_info->gnome_win);
-    grabKey (dpy, &screen_info->params->keys[KEY_WORKSPACE_4], screen_info->gnome_win);
-    grabKey (dpy, &screen_info->params->keys[KEY_WORKSPACE_5], screen_info->gnome_win);
-    grabKey (dpy, &screen_info->params->keys[KEY_WORKSPACE_6], screen_info->gnome_win);
-    grabKey (dpy, &screen_info->params->keys[KEY_WORKSPACE_7], screen_info->gnome_win);
-    grabKey (dpy, &screen_info->params->keys[KEY_WORKSPACE_8], screen_info->gnome_win);
-    grabKey (dpy, &screen_info->params->keys[KEY_WORKSPACE_9], screen_info->gnome_win);
-    grabKey (dpy, &screen_info->params->keys[KEY_SHORTCUT_1], screen_info->gnome_win);
-    grabKey (dpy, &screen_info->params->keys[KEY_SHORTCUT_2], screen_info->gnome_win);
-    grabKey (dpy, &screen_info->params->keys[KEY_SHORTCUT_3], screen_info->gnome_win);
-    grabKey (dpy, &screen_info->params->keys[KEY_SHORTCUT_4], screen_info->gnome_win);
-    grabKey (dpy, &screen_info->params->keys[KEY_SHORTCUT_5], screen_info->gnome_win);
-    grabKey (dpy, &screen_info->params->keys[KEY_SHORTCUT_6], screen_info->gnome_win);
-    grabKey (dpy, &screen_info->params->keys[KEY_SHORTCUT_7], screen_info->gnome_win);
-    grabKey (dpy, &screen_info->params->keys[KEY_SHORTCUT_8], screen_info->gnome_win);
-    grabKey (dpy, &screen_info->params->keys[KEY_SHORTCUT_9], screen_info->gnome_win);
-    grabKey (dpy, &screen_info->params->keys[KEY_SHORTCUT_10], screen_info->gnome_win);
-    grabKey (dpy, &screen_info->params->keys[KEY_UP_WORKSPACE], screen_info->gnome_win);
-    grabKey (dpy, &screen_info->params->keys[KEY_DOWN_WORKSPACE], screen_info->gnome_win);
-    grabKey (dpy, &screen_info->params->keys[KEY_LEFT_WORKSPACE], screen_info->gnome_win);
-    grabKey (dpy, &screen_info->params->keys[KEY_RIGHT_WORKSPACE], screen_info->gnome_win);
-
+    myScreenGrabKeys (screen_info);
 
     return TRUE;
 }
@@ -1074,7 +1044,7 @@ gboolean
 loadSettings (ScreenInfo *screen_info)
 {
     Settings rc[] = {
-        /* Do not chnage the order of the following parameters */
+        /* Do not change the order of the following parameters */
         {"active_text_color", NULL, FALSE},
         {"inactive_text_color", NULL, FALSE},
         {"active_border_color", NULL, FALSE},
@@ -1470,14 +1440,14 @@ initSettings (ScreenInfo *screen_info)
         workspaceSetCount (screen_info, val);
     }
 
-    if (getUTF8String (display_info, screen_info->xroot, NET_DESKTOP_NAMES, &names, &i))
+    if (getUTF8StringList (display_info, screen_info->xroot, NET_DESKTOP_NAMES, &names, &i))
     {
         workspaceSetNames (screen_info, names, i);
     }
     else
     {
         screen_info->workspace_names = NULL;
-        screen_info->workspace_names_length = 0;
+        screen_info->workspace_names_items = 0;
     }
 
     getDesktopLayout(display_info, screen_info->xroot, screen_info->workspace_count, &screen_info->desktop_layout);
