@@ -1,4 +1,4 @@
-/*      $Id: workspaces.c 17585 2005-09-13 20:22:54Z olivier $
+/*      $Id: workspaces.c 20529 2006-03-24 14:46:19Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -376,15 +376,15 @@ workspaceSwitch (ScreenInfo *screen_info, int new_ws, Client * c2)
 }
 
 void
-workspaceSetNames (ScreenInfo * screen_info, char *names, int length)
+workspaceSetNames (ScreenInfo * screen_info, char **names, int items)
 {
     if (screen_info->workspace_names)
     {
-        g_free (screen_info->workspace_names);
+        g_strfreev (screen_info->workspace_names);
     }
 
     screen_info->workspace_names = names;
-    screen_info->workspace_names_length = length;
+    screen_info->workspace_names_items = items;
 }
 
 void
