@@ -1,4 +1,4 @@
-/*      $Id: workspaces.h,v 1.21 2004/09/23 05:59:37 olivier Exp $
+/*      $Id: workspaces.h 20529 2006-03-24 14:46:19Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -29,12 +29,13 @@
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Xmd.h>
+#include <glib.h>
 #include "screen.h"
 #include "client.h"
 
 gboolean workspaceMove (ScreenInfo *, int, int, Client *);
 void workspaceSwitch (ScreenInfo *, int, Client *);
-void workspaceSetNames (ScreenInfo *, char *, int);
+void workspaceSetNames (ScreenInfo *, char **, int);
 void workspaceSetCount (ScreenInfo *, int);
 void workspaceUpdateArea (ScreenInfo *);
 
